/**********************************************************************
 * $Id: Bsb.java 8765 2006-11-15 00:08:29Z jjdelcerro $
 *
 * Name:     Bsb.java
 * Project:  JGDAL. Interface java to gdal (Frank Warmerdam).
 * Purpose:  Funcions to Open, Close and read data of bsb files. 
 * Author:   Nacho Brodin, brodin_ign@gva.es
 *
 **********************************************************************/
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/

package es.gva.cit.jbsb;

import java.io.*;
import java.util.Date;



/**
 * Ofrece las funcionalidades de la librer&iacute;a BSB de C a las
 * aplicaciones de java.
 * 
 * @author Nacho Brodin <brodin_ign@gva.es>
 */


public class Bsb {
	
	/**
	 *Objeto que contiene la cabecera del fichero que ha sido cargada
	 *desde la librer&iacute;a de C. 
	 */
	
	public BsbInfo g_psInfo = null;
	
	/**
	 *Objeto que contiene el buffer para una l&iacute;nea que sea le&iacute;da
	 *desde el fichero bsb. 
	 */
	
	public BsbBuffer g_buffer = null;
	
	/**
	 *Variable que controla que BSBReadLine llame a BSBReadInfoLineNat (1) o a 
	 *BSBReadLineNat (0). 
	 * 
	 */
	public long cPtr;
	
	
	/**
	 * Nombre del fichero abierto
	 */
	
	private String id;
	
		 
	//Definicin de mtodos nativos 
	//********************************************************
	
	
	private native BsbInfo bsbOpenNat(String pszFilename);
	
	
	/**
	 * Lee la l&iacute;nea del fichero bsb indicada por nScanline manteniendo la BSBInfo sin retornar
	 * al cliente java. Esto hace que la llamada tenga un menor coste computacional pero que
	 * no sea posible acceder a los datos modificados en cada llamada por la funci&oacute;n de libreria 
	 * de C BSBReadScanLine.
	 * 	
	 * @param buf	Buffer que contiene la l&iacute;nea le&iacute;da en cada llamada
	 * @param nScanline	N&uacute;mero de l&iacute;nea procesada
	 * 
	 * @return 0 si hay error y 1 si la lectura de l&iacute;nea ha sido correcta
	 */
	
	private native int bsbReadLineNat( 	long cPtr,
											BsbBuffer buf, 
											int nScanline,
											String pszFilename);
	
	
    private native void bsbCloseNat( long cPtr);

        
    //Recubrimiento de los mtodos nativos e interfaz exterior 
    //********************************************************    
    
    
    /**
     *Abre el fichero de imgen 
     *
     *@param pszFilename	Nombre del fichero bsb
     */
    
    public void bsbOpen(String fil)throws BsbException,IOException{
    	
    	//Comprobaciones
    	
    	File f = new File( fil );
		if(!f.exists())
		  throw new IOException("El archivo "+fil+" no existe");
		  
	    if(!f.canRead())
	      throw new IOException("El archivo no puede leerse");
    	
    	//Abrimos el fichero y obtenemos la cabecera
    	
    	g_psInfo = bsbOpenNat(fil);
    	if(g_psInfo.cPtr<=0)
	    	throw new BsbException("Error en Open de BSB");
    	
    	//Creamos el buffer
    	
    	g_buffer = new BsbBuffer();
    	g_buffer.reserva(g_psInfo.nXSize);
    }
    
    
    /**
     *Lee una lnea del fichero de imagen
     *
     *@param sScanline -> Nmero de lnea a procesar
     *@param flag	0: No devuelve la estructura BSBInfo
     *				1: Devuelve a java la BSBInfo modificado 
     */
    
  
    public void bsbReadLine(int nScanline)throws BsbException{
    	int res = 0;
    	
    	if(g_psInfo.cPtr <= 0)
			throw new BsbException("Error en BSBReadLine(). La llamada BSBOpen no tuvo exito");
    	
    	res = bsbReadLineNat(g_psInfo.cPtr,g_buffer,nScanline,id);
  
    	
    	if(res == 0)throw new BsbException();
    
    }
    
    
    
    /**
     *Cierra el fichero de imgen y elimina la cabecera 
     */
    
    public void bsbClose()throws BsbException{
    	
    	if(g_psInfo.cPtr <= 0)
			throw new BsbException("Error en BSBClose(). La llamada BSBOpen no tuvo exito");
    	
    	bsbCloseNat(g_psInfo.cPtr);
    }
    
    
	static{
		
		System.loadLibrary("jgdal");
		
	} 
}


